@@ -58,13 +58,14 @@ from tg.wsgiapp import TGApp
 from tg.controllers import TGController, redirect, url, lurl, abort
 from tg.configuration import config
 from tg.release import version
-from tg.decorators import (validate, expose, override_template,
-                           require, allow_only)
+from tg.decorators import (validate, expose, override_template, use_custom_format,
+                           require, allow_only, with_engine)
 
 from tg.flash import flash, get_flash, get_status
 from tg.jsonify import encode as json_encode
 from tg.controllers.util import use_wsgi_app
 from tg.controllers.dispatcher import dispatched_controller
+from tg.render import render as render_template
 
 __version__ = version
 
@@ -72,4 +73,4 @@ __all__ = ['__version__',
     'allow_only', 'app_globals', 'expose', 'override_template', 'request',
     'require', 'response', 'session', 'TGApp', 'TGController', 'tmpl_context',
     'use_wsgi_app', 'validate', 'i18n','json_encode', 'cache', 'url', 'lurl',
-    'dispatched_controller']
+    'dispatched_controller', 'use_custom_format', 'with_engine', 'render_template']
